from typing import TYPE_CHECKING

from based_utils.calx import fractions
from based_utils.cli import Colored
from based_utils.colors import Color, ColorTheme

if TYPE_CHECKING:
    from collections.abc import Iterable, Iterator


def _color_shade(color: Color) -> str:
    return Colored(color.as_hex.center(8), color.contrasting_shade, color).formatted


def _color_line(color: Color, name: str, shades: Iterable[float]) -> str:
    shades_str = "".join(_color_shade(color.shade(s)) for s in shades)
    hue = "   " if name == "grey" else f"{color.hue * 360:03.0f}"
    return shades_str + Colored(f" {hue} {name}", color).formatted


def color_lines(color_theme: dict[str, Color]) -> Iterator[str]:
    shades = list(fractions(11))

    # Any color will be grey when saturation is set to 0.
    yield _color_line(ColorTheme.grey, "grey", shades)

    # Shade/hue tables for specific saturation values
    num_saturations = 4
    for i in range(1, num_saturations + 1):
        # Shade percentages row
        yield "".join(f"{s:.2%}".center(8) for s in shades)
        # Tables
        saturation = i / num_saturations
        for name, color in sorted(color_theme.items(), key=lambda p: p[1].hue):
            yield _color_line(color.adjust(saturation=saturation), name, shades)



import argparse

from based_utils.cli import LogLevel, check_integer_in_range, parse_key_value_pair
from based_utils.colors import AltColors, Color, Colors, c
from based_utils.data import get_class_vars, try_convert

from . import log
from .color_theme import color_lines
from .css_variables import ShadesParams, shades_as_css_variables

_log = log.get_logger()


def _css_variables(args: argparse.Namespace) -> None:
    with log.context(LogLevel.INFO):
        c1 = Color.from_hex(args.color1)
        c2 = Color.from_hex(args.color2) if args.color2 else None
        params = ShadesParams(
            args.amount,
            args.dynamic_range / 100,
            args.include_black_and_white,
            args.include_input_shades,
        )
        for line in shades_as_css_variables(c1, c2, params=params, label=args.label):
            _log.info(line)


def parse_args_css_variables(parser: argparse.ArgumentParser) -> None:
    parser.add_argument("-l", "--label", type=str, default="color")
    parser.add_argument("-c", "--color1", type=str, required=True)
    parser.add_argument("-k", "--color2", type=str)
    parser.add_argument(
        "-n", "--amount", type=check_integer_in_range(0, None), default=19
    )
    parser.add_argument(
        "-b", "--include-black-and-white", action="store_true", default=False
    )
    parser.add_argument(
        "-i", "--include-input-shades", action="store_true", default=False
    )
    parser.add_argument(
        "-d", "--dynamic-range", type=check_integer_in_range(0, 100), default=0
    )
    parser.set_defaults(func=_css_variables)


def _color_theme(args: argparse.Namespace) -> None:
    hues = {k: c(try_convert(int, h, default=333)) for k, h in args.color_hues}
    if args.merge_with_default_theme or not hues:
        theme_cls = AltColors if args.alt_default_theme else Colors
        hues = dict(get_class_vars(theme_cls, Color)) | hues
    with log.context(LogLevel.INFO):
        for line in color_lines(hues):
            _log.info(line)


def parse_args_color_theme(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "-c",
        "--color-hues",
        nargs="+",
        metavar="NAME=HUE (1-360)",
        type=parse_key_value_pair,
        default={},
    )
    parser.add_argument(
        "-m", "--merge-with-default-theme", action="store_true", default=False
    )
    parser.add_argument("-a", "--alt-default-theme", action="store_true", default=False)
    parser.set_defaults(func=_color_theme)


def main() -> None:
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(required=True)
    parse_args_css_variables(subparsers.add_parser("css"))
    parse_args_color_theme(subparsers.add_parser("colors"))
    args = parser.parse_args()
    args.func(args)



from dataclasses import dataclass
from typing import TYPE_CHECKING

from based_utils.calx import InterpolationBounds, fractions, interpolate
from based_utils.cli import Colored

if TYPE_CHECKING:
    from collections.abc import Iterator

    from based_utils.colors import Color


def _colored(s: str, color: Color) -> str:
    return Colored(s, color.contrasting_shade, color).formatted


def _css_color_comment(color: Color) -> str:
    return _colored(f"#{color.as_hex} --> {color}", color)


@dataclass(frozen=True)
class ShadesParams:
    amount: int = 19
    dynamic_range: float = 0
    include_black_white: bool = False
    include_input: bool = False


def _shades(
    c_dark: Color, c_bright: Color, *, params: ShadesParams
) -> list[tuple[Color, int]]:
    old_li_dark, old_li_bright = c_dark.lightness, c_bright.lightness
    li_dark = interpolate(params.dynamic_range, (old_li_dark, 0))
    li_bright = interpolate(params.dynamic_range, (old_li_bright, 1))
    dark, bright = c_dark.shade(li_dark), c_bright.shade(li_bright)
    li_bounds = InterpolationBounds(li_dark, li_bright)

    def color_for_li(lightness: float) -> Color:
        return dark.blend(bright, li_bounds.inverse_interpolate(lightness))

    shades = fractions(params.amount, inclusive=params.include_black_white)
    colors = [(color_for_li(li), 0) for li in shades]
    if params.include_input:
        colors += [(color_for_li(old_li_dark), 1), (color_for_li(old_li_bright), 2)]
    return sorted(colors)


def shades_as_css_variables(
    c_1: Color, c_2: Color | None, *, params: ShadesParams, label: str
) -> Iterator[str]:
    yield "/*"
    yield "Based on:"

    if c_2:
        if c_2 < c_1:
            c_1, c_2 = c_2, c_1

        yield f"- Darkest:   {_css_color_comment(c_1)}"
        yield f"- Brightest: {_css_color_comment(c_2)}"
        shades = _shades(c_1, c_2, params=params)

    else:
        yield _css_color_comment(c_1)
        shades = [
            (c_1.shade(s), 0)
            for s in fractions(params.amount, inclusive=params.include_black_white)
        ]
        if params.include_input:
            shades.append((c_1, 1))
        shades.sort()

    yield "*/"

    for c, v in shades:
        if v:
            s = " <-- "
            if c_2:
                s += f"same lightness as {'darkest' if v == 1 else 'brightest'} "
            s += "input"
            c_print = c.contrasting_shade

        else:
            s, c_print = "", c

        var = f"{f'--{label}-{c.lightness * 100:02.0f}'}: #{c.as_hex};"
        comment = f"/* --> {c}{s} */"
        yield _colored(f"{var} {comment}", c_print)
